from enum import Enum, auto

class ConnTypeEnum(Enum):
    COORD = auto()
    LOGIC = auto()
    SEMI = auto()
    QUASI = auto()
    CONJ = auto()
    DISJ = auto()
    SIMPLE = auto()
    COMPLEX = auto()
    COORDCONJ = auto()
    ADDITION = auto()
    CONTRAST = auto()
    LINK = auto()
    ORDER = auto()
    REFERENCE = auto()
    REASONANDPURPOSE = auto()
    COND = auto()
    CONCC = auto()
    OPPOSIT = auto()
    TEMPORAL = auto()
    CONJUNCTS = auto()
    
